package com.froglogic.extension.ktable;

import java.lang.reflect.Method;
import java.util.AbstractList;

import com.froglogic.squish.extension.*;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.ScrollBar;

import de.kupzog.ktable.KTable;
import de.kupzog.ktable.KTableModel;


class CellInspectable extends InspectableAdapter
{
    Cell cell_proxy;

    CellInspectable( Cell cp )
    {
        cell_proxy = cp;
    }
    public Object getObject()
    {
        return cell_proxy;
    }
    public Object getParent()
    {
        return cell_proxy.ktable;
    }
    public Rect getGlobalBounds()
    {
        org.eclipse.swt.graphics.Point p = cell_proxy.ktable.getSize();
        Rectangle r1 = Display.getCurrent().map( cell_proxy.ktable, null,
                0, 0, p.x, p.y );
        Rectangle r2 = cell_proxy.ktable.getCellRect(
                cell_proxy.column, cell_proxy.row);
        return new Rect( r1.x + r2.x, r1.y + r2.y, r2.width, r2.height );
    }
    public Point mapToGlobal( Point pos )
    {
        pos = super.mapToGlobal( pos );
        return pos;
    }
    public Object getPropertyValue( String prop )
    {
        if ( prop.equals( "container" ) ) {
            return cell_proxy.ktable;
        } else if ( prop.equals( "row" ) ) {
            return cell_proxy.row;
        } else if ( prop.equals( "column" ) ) {
            return cell_proxy.column;
        }
        return null;
    }
    private static void scroll( ScrollBar sb, int pos )
    {
        if ( sb != null && sb.getVisible() ) {
            sb.setSelection( pos );
            Display display = Display.getCurrent();
            Event event = new Event();
            event.display = display;
            event.widget = sb;
            event.type = SWT.Selection;
            try {
                Method post = Display.class.getDeclaredMethod( "postEvent", new Class[]{ Event.class } );
                post.setAccessible( true );
                post.invoke( display, new Object[]{ event  } );
            } catch ( Exception ex ) {
                System.err.println( "Error postEvent " + ex.getMessage() );
            }
        }
    }
    public boolean isObjectReady()
    {
        KTableModel model = cell_proxy.ktable.getModel();
        if ( cell_proxy.column < model.getColumnCount()
                && cell_proxy.column >= model.getFixedHeaderColumnCount()
                && cell_proxy.row < model.getRowCount()
                && cell_proxy.row >= model.getFixedHeaderRowCount() )
        {
            boolean scrolled = false;
            Rectangle r = cell_proxy.ktable.getVisibleCells();
            if ( r.x > cell_proxy.column || r.x + r.width < cell_proxy.column ) {
                scroll( cell_proxy.ktable.getHorizontalBar(), cell_proxy.column );
                scrolled = true;
            }
            if ( r.y > cell_proxy.row || r.y + r.height < cell_proxy.row ) {
                scroll( cell_proxy.ktable.getVerticalBar(), cell_proxy.row );
                scrolled = true;
            }

            if ( scrolled )
                return false;
        }
        return super.isObjectReady();
    }
}


class KTableInspectable extends InspectableAdapter
{
    KTable ktable;

    KTableInspectable( KTable kt )
    {
        ktable = kt;
    }
    public Object getObject()
    {
        return ktable;
    }
    public Object getParent()
    {
        return ktable.getParent();
    }
    public Object getChildAt( Point pos )
    {
        org.eclipse.swt.graphics.Point p = ktable.calcColumnNum( pos.x, pos.y );
        if ( p.x >= 0 && p.y >= 0 )
            return new Cell( ktable, p.y, p.x );
        return null;
    }
    public Rect getGlobalBounds()
    {
        org.eclipse.swt.graphics.Point p = ktable.getSize();
        Rectangle r = Display.getCurrent().map( ktable, null, 0, 0, p.x, p.y );
        return new Rect( r.x, r.y, r.width, r.height );
    }
    protected boolean findObjects( ObjectQuery query, int n,
            Object searchee, AbstractList collect )
    {
        String type = query.getStringProperty( "type" );
        Object obj = getObject();

        if ( obj == searchee ) {
            collect.add( obj );
            return true;
        }

        if ( obj.getClass().getName().equals( type ) && query.match( obj ) ) {
            collect.add( obj );
            if ( n > 0 && collect.size() == n )
                return true;
        }

        // Cell must have KTable set as container, row and column set
        if ( !type.equals( "com.froglogic.extension.ktable.Cell" ) ||
                !query.hasProperty( "container" ) ||
                !query.hasProperty( "row" ) ||
                !query.hasProperty( "column" ) ) {
            return false;
        }

        Object con = query.getObjectProperty( "container" );
        if ( !( con instanceof KTable ) )
            return false;

        int row = query.getIntProperty( "row" );
        int column = query.getIntProperty( "column" );
        Cell cp = new Cell( (KTable)con, row, column );

        if ( searchee != null ) {
            if ( cp.equals( searchee ) ) {
                collect.add( cp );
                return true;
            }
        } else if ( query.match( cp ) ) {
            collect.add( cp );
            if ( n > 0 && collect.size() == n )
                return true;
        }
        return false;
    }
}


public class KTableFactory implements InspectableFactory
{
    public Inspectable query( Object object )
    {
        if ( object instanceof KTable ) {
            return new KTableInspectable( (KTable)object );
        } else if ( object instanceof Cell ) {
            return new CellInspectable( (Cell)object );
        }
        return null;
    }
    public int priority()
    {
        return InspectableFactory.NORM_PRIORITY;
    }
    public static void init( InspectableRegistry registry )
    {
        registry.register( new KTableFactory() );
    }
}
